#include <jni.h>
#include <cstdio>
#include <cstdlib>
#include <unistd.h>
#include <fcntl.h>
#include <csignal>
#include <cstring>
#include "tuya_iot_base_api.h"
#include "uni_log.h"
#include <android/log.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define LOG_TAG "tuya_iot_init"
#define LOGI(...) __android_log_print(ANDROID_LOG_INFO, LOG_TAG, __VA_ARGS__)
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)
#define LOGW(...) __android_log_print(ANDROID_LOG_WARN, LOG_TAG, __VA_ARGS__)
#define LOGV(...) __android_log_print(ANDROID_LOG_VERBOSE, LOG_TAG, __VA_ARGS__)


// TuyaOS获取到短链接之后调用此接口输出qrcode打印
STATIC VOID __qrcode_active_shourturl_cb(CONST CHAR_T *shorturl)
{
    if (NULL == shorturl) {
        return;
    }
    LOGD("shorturl : %s", shorturl);
    ty_cJSON *item = ty_cJSON_Parse(shorturl);
//    __qrcode_printf();
    LOGD("shorturl : %s", ty_cJSON_GetObjectItem(item, "shortUrl")->valuestring);
    ty_cJSON_Delete(item);

    return;
}

STATIC VOID_T __soc_dev_status_changed_cb(IN CONST GW_STATUS_E status)
{
    LOGD("soc device status: %d", status);
    return;
}

// 回调函数原型示例
static void network_status_handler(GW_BASE_NW_STAT_T status) {
    switch(status) {
        case GB_STAT_CLOUD_CONN:
            // 云端连接成功，开始同步数据
            LOGD("network_status_handler GB_STAT_CLOUD_CONN");
            break;
        case GB_STAT_LAN_CONN:
            // 局域网连接成功，启用本地控制
            LOGD("network_status_handler GB_STAT_LAN_CONN");
            break;
        case GB_STAT_LAN_UNCONN:
            // 网络断开，启动重连机制
            LOGD("network_status_handler GB_STAT_LAN_UNCONN");;
            break;
    }
}

bool resolveDomainIP(JNIEnv *env) {
    struct addrinfo hints = {0}, *res = nullptr;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET; // 仅解析 IPv4
    hints.ai_socktype = SOCK_STREAM;

    int dnsResult = getaddrinfo("h6.iot-dns.com", nullptr, &hints, &res);
    if (dnsResult != 0 || res == nullptr) {
        LOGE("解析 h6.iot-dns.com 域名失败: %s", gai_strerror(dnsResult));
        freeaddrinfo(res);
        return false;
    }

    struct sockaddr_in *addr = (struct sockaddr_in *)res->ai_addr;
    char ip[INET_ADDRSTRLEN];
    inet_ntop(AF_INET, &(addr->sin_addr), ip, sizeof(ip));
    LOGD("h6.iot-dns.com 解析 IP 地址: %s", ip);
    freeaddrinfo(res);
    return true;
}

bool initDatabaseAndLogPath(JNIEnv *env, jstring path, jstring log_path, OPERATE_RET *rt) {
    auto getUtfString = [&](jstring str) -> const char* {
        if (str == nullptr) return nullptr;
        return env->GetStringUTFChars(str, nullptr);
    };

    auto releaseUtfString = [&](jstring str, const char* utfStr) {
        if (str != nullptr && utfStr != nullptr) {
            env->ReleaseStringUTFChars(str, utfStr);
        }
    };

    const char* pathStr = getUtfString(path);
    if (pathStr == nullptr) {
        LOGE("无法获取 path 字符串");
        return false;
    }

    TY_INIT_PARAMS_S params = {
            .init_db = true,
    };

    const char* logPathStr = getUtfString(log_path);
    if (logPathStr == nullptr) {
        LOGE("无法获取 log_path 字符串");
        releaseUtfString(path, pathStr);
        return false;
    }

    strncpy(params.log_seq_path, logPathStr, LOG_SEQ_PATH_LEN - 1);
    params.log_seq_path[LOG_SEQ_PATH_LEN - 1] = '\0';

    *rt = tuya_iot_init_params(pathStr, &params);
    LOGD("tuya_iot_init_params result:%d", *rt);
    if (*rt != OPRT_OK) {
        LOGE("tuya_iot_init_params result:%d", *rt);
        releaseUtfString(path, pathStr);
        releaseUtfString(log_path, logPathStr);
        return false;
    }

    releaseUtfString(path, pathStr);
    releaseUtfString(log_path, logPathStr);
    return true;
}

bool setProductInfo(JNIEnv *env, jstring uuid, jstring authKey, OPERATE_RET *rt) {
    auto getUtfString = [&](jstring str) -> const char* {
        if (str == nullptr) return nullptr;
        return env->GetStringUTFChars(str, nullptr);
    };

    auto releaseUtfString = [&](jstring str, const char* utfStr) {
        if (str != nullptr && utfStr != nullptr) {
            env->ReleaseStringUTFChars(str, utfStr);
        }
    };

    const char* uuidStr = getUtfString(uuid);
    const char* authKeyStr = getUtfString(authKey);
    if (uuidStr == nullptr || authKeyStr == nullptr) {
        LOGE("无法获取 uuid 或 authKey 字符串");
        releaseUtfString(uuid, uuidStr);
        releaseUtfString(authKey, authKeyStr);
        return false;
    }

    GW_PROD_INFO_S prod_info = {
            .uuid = strdup(uuidStr),
            .auth_key = strdup(authKeyStr),
    };

    *rt = tuya_iot_set_gw_prod_info(&prod_info);
    LOGD("tuya_iot_set_gw_prod_info result:%d", *rt);
    if (*rt != OPRT_OK) {
        LOGE("tuya_iot_set_gw_prod_info err:%d", *rt);
        free((void*)prod_info.uuid);
        free((void*)prod_info.auth_key);
        releaseUtfString(uuid, uuidStr);
        releaseUtfString(authKey, authKeyStr);
        return false;
    }

    free((void*)prod_info.uuid);
    free((void*)prod_info.auth_key);
    releaseUtfString(uuid, uuidStr);
    releaseUtfString(authKey, authKeyStr);
    return true;
}

bool registerCallbacks(JNIEnv *env, jstring pid, OPERATE_RET *rt) {
    auto getUtfString = [&](jstring str) -> const char* {
        if (str == nullptr) return nullptr;
        return env->GetStringUTFChars(str, nullptr);
    };

    auto releaseUtfString = [&](jstring str, const char* utfStr) {
        if (str != nullptr && utfStr != nullptr) {
            env->ReleaseStringUTFChars(str, utfStr);
        }
    };

    const char* pidStr = getUtfString(pid);
    if (pidStr == nullptr) {
        LOGE("无法获取 pid 字符串");
        return false;
    }

    TY_IOT_CBS_S iot_cbs = {};
    iot_cbs.active_shorturl = __qrcode_active_shourturl_cb;
    iot_cbs.gw_status_cb = __soc_dev_status_changed_cb;

    *rt = tuya_iot_soc_init_param(&iot_cbs, pidStr, nullptr, "1.0.0");
    LOGD("tuya_iot_soc_init_param result:%d", *rt);
    releaseUtfString(pid, pidStr);

    *rt = tuya_iot_reg_get_nw_stat_cb(network_status_handler);
    if (*rt != OPRT_OK) {
        LOGE("tuya_iot_reg_get_nw_stat_cb err:%d", *rt);
    }

    return true;
}


extern "C" JNIEXPORT jint JNICALL
Java_com_zszn_iot_tuya_IotJniApi_tuyaInit(JNIEnv *env, jobject thiz, jstring path, jstring log_path, jstring pid, jstring uuid, jstring authKey) {
    OPERATE_RET rt = OPRT_COM_ERROR;

    // 解析域名IP地址
//    if (!resolveDomainIP(env)) {
//        return -1;
//    }

    // 初始化数据库和设置日志路径
    if (!initDatabaseAndLogPath(env, path, log_path, &rt)) {
        return -1;
    }

    // 设置产品信息
    if (!setProductInfo(env, uuid, authKey, &rt)) {
        return -2;
    }

    // 注册回调函数
    if (!registerCallbacks(env, pid, &rt)) {
        return rt;
    }

    return rt;
}

// 网络状态回调函数
void __soc_network_status_cb(GW_BASE_NW_STAT_T status) {
    switch(status) {
        case GB_STAT_CLOUD_CONN:
            __android_log_print(ANDROID_LOG_VERBOSE, "tuya_iot_init", "云端连接成功，开始同步数据");
            break;
        case GB_STAT_LAN_CONN:
            __android_log_print(ANDROID_LOG_VERBOSE, "tuya_iot_init", "局域网已连接，开始本地控制");
            break;
        case GB_STAT_LAN_UNCONN:
            __android_log_print(ANDROID_LOG_VERBOSE, "tuya_iot_init", "网络断开，启动重连机制");
            break;
    }
}
